<?php

if (!defined('_PS_VERSION_'))
	exit;

class FreeShipping extends CarrierModule 
{
	private $_html = '';
	private $_postErrors = array();

	private $free_shipping_bound = 200;
	private $free_shipping_weight_bound = 0;
	private $exclude_cat = array();
	private $exclude_prd = array();
	private $include_prd = array();

	private $currency;
	private $weight_unit;

	public function __construct() {
		$this->name = 'freeshipping';
		$this->tab = 'shipping_logistics';
		$this->version = '1.0';
		$this->author = 'Alexander Brakowski';
		$this->limited_countries = array('de');

		parent::__construct ();

		$this->displayName = $this->l('Kostenloser Versand');
		$this->description = $this->l('Spezifiziere die Vorraussetzungen für einen kostenlosen Versand.');

		$this->loadVars();

	}

	private function loadVars(){
		if(!self::isInstalled($this->name)) return;

		$this->free_shipping_bound = floatval(Configuration::get('FREE_SHIPPING_BOUNDARY'));
		$this->free_shipping_weight_bound = floatval(Configuration::get('FREE_SHIPPING_WEIGHT_BOUNDARY'));
		$this->exclude_cat = unserialize(Configuration::get('FREE_SHIPPING_EXCLUDE_CAT'));
		$this->exclude_prd = unserialize(Configuration::get('FREE_SHIPPING_EXCLUDE_PRD'));
		$this->include_prd = unserialize(Configuration::get('FREE_SHIPPING_INCLUDE_PRD'));

		$this->currency = new Currency(Configuration::get('PS_CURRENCY_DEFAULT'));
		$this->weight_unit = Configuration::get('PS_WEIGHT_UNIT');
	}

	public function install(){
		Configuration::updateValue('FREE_SHIPPING_BOUNDARY', 200);
		Configuration::updateValue('FREE_SHIPPING_WEIGHT_BOUNDARY', 0);
		Configuration::updateValue('FREE_SHIPPING_EXCLUDE_CAT', serialize(array()));
		Configuration::updateValue('FREE_SHIPPING_EXCLUDE_PRD', serialize(array()));
		Configuration::updateValue('FREE_SHIPPING_INCLUDE_PRD', serialize(array()));
		
		// Install Module
		if (!parent::install() || !$this->installCarrier())
			return false;

		return true;
	}

	public function uninstall(){
		// Uninstall Carriers
		Db::getInstance()->autoExecute(_DB_PREFIX_.'carrier', array('deleted' => 1), 'UPDATE', '`external_module_name` = \'freeshipping\'');

		// Uninstall Module
		if (!parent::uninstall())
			return false;

		return true;
	}

	private function _in_array($arr, $haystack){
		foreach($arr as $a){
			if(in_array($a, $haystack)) return true;
		}

		return false;
	}

	public function getOrderShippingCost($cart, $shipping_cost) {
		if(($this->free_shipping_bound > 0 && $cart->getOrderTotal(true, Cart::BOTH_WITHOUT_SHIPPING) >= $this->free_shipping_bound) || 
		   ($this->free_shipping_weight_bound > 0 && $cart->getTotalWeight() >= $this->free_shipping_weight_bound)){

			$products = $cart->getProducts();
			foreach($products as $p){
				$cats = Product::getProductCategories($p['id_product']);

				if(($this->_in_array($cats, $this->exclude_cat) ||
				   in_array($p['id_product'], $this->exclude_prd)) &&
				   !in_array($p['id_product'], $this->include_prd)) 
				   		return $shipping_cost;
			}

			return 0;
		}

		return $shipping_cost;
	}

	public function getOrderShippingCostExternal($params) {
		return $this->getOrderShippingCost($params, 0);
	}

	public function installCarrier()
	{
		$carrier = new Carrier();
		$carrier->name = $this->l("Kostenloser Versand");
		$carrier->id_tax_rules_group = 0;
		$carrier->id_zone = 1;
		$carrier->active = true;
		$carrier->deleted = 0;
		$carrier->delay = array();
		$carrier->shipping_handling = false;
		$carrier->range_behavior = 0;
		$carrier->is_module = true;
		$carrier->shipping_external = true;
		$carrier->external_module_name = 'freeshipping';
		$carrier->need_range = true;
		$carrier->shipping_method = 2;

		$languages = Language::getLanguages(true);
		foreach ($languages as $language){
			$carrier->delay[(int)$language['id_lang']] = $carrier->name;
		}

		if ($carrier->add())
		{
			$groups = Group::getGroups(true);
			foreach ($groups as $group)
				Db::getInstance()->autoExecute(_DB_PREFIX_.'carrier_group', array('id_carrier' => (int)($carrier->id), 'id_group' => (int)($group['id_group'])), 'INSERT');

			$rangeWeight = new RangeWeight();
			$rangeWeight->id_carrier = $carrier->id;
			$rangeWeight->delimiter1 = '0';
			$rangeWeight->delimiter2 = '10000';
			$rangeWeight->add();

			$zones = Zone::getZones(true);
			foreach ($zones as $zone)
			{
				Db::getInstance()->autoExecute(_DB_PREFIX_.'carrier_zone', array('id_carrier' => (int)($carrier->id), 'id_zone' => (int)($zone['id_zone'])), 'INSERT');
				Db::getInstance()->autoExecuteWithNullValues(_DB_PREFIX_.'delivery', array('id_carrier' => (int)($carrier->id), 'id_range_price' => (int)($rangePrice->id), 'id_range_weight' => NULL, 'id_zone' => (int)($zone['id_zone']), 'price' => '0'), 'INSERT');
				Db::getInstance()->autoExecuteWithNullValues(_DB_PREFIX_.'delivery', array('id_carrier' => (int)($carrier->id), 'id_range_price' => NULL, 'id_range_weight' => (int)($rangeWeight->id), 'id_zone' => (int)($zone['id_zone']), 'price' => '0'), 'INSERT');
			}

			// Copy Logo
			if (!copy(dirname(__FILE__).'/carrier.jpg', _PS_SHIP_IMG_DIR_.'/'.(int)$carrier->id.'.jpg'))
				return false;

			// Return ID Carrier
			return (int)($carrier->id);
		}

		return false;
	}

	private function makeRevCatOption($arr, $curr, $id, &$out){
		$out .= '<option value="' . $id . '">' . str_repeat('|&nbsp;&nbsp;&nbsp;&nbsp;', $curr['infos']['level_depth']) . $curr['infos']['name'] . '</option>';
		if(isset($arr[$id])){
			foreach($arr[$id] as $key => $sub){
				$this->makeRevCatOption($arr, $arr[$id][$key], $key, $out);
			}
		}
	}
	
	private function makeMenu($type){
		global $cookie;
		
		$_menu = '<select name="'.$type.'id"><option value="">---</option>';
		if($type == "cat"){
			$_list = Category::getCategories(intval($cookie->id_lang), false);
			$this->makeRevCatOption($_list, $_list[0][1], 1, $_menu);
		}
		$_menu .= '</select>';
		return $_menu;
	}

	public function _displayForm() {
		

		$this->_html .= '
		<form action="'.$this->getConfUrl().'" method="post">
			<input type="hidden" name="post_action" value="save_settings" />
			<fieldset><legend><img src="../img/admin/edit.gif" alt="" title="" />'.$this->l('Einstellungen').'</legend>
				<label>'.$this->l('Ab Gewicht').'</label>
				<div class="margin-form"><input type="text" name="from-weight" size="5" value="'.$this->free_shipping_weight_bound.'" /> '.$this->weight_unit.'
					<p class="clear">'.$this->l('Kostenloser Versand ab definierten Gewicht. "0" angeben, um Funktion zu deaktivieren.').'</p>
				</div>
				<label>'.$this->l('Ab Betrag').'</label>
				<div class="margin-form"><input type="text" name="from-money" size="5" value="'.$this->free_shipping_bound.'" /> '.$this->currency->getSign().'
					<p class="clear">'.$this->l('Kostenloser Versand ab definierten Betrag. "0" angeben, um Funktion zu deaktivieren.').'</p>
				</div>
				<center><input type="submit" name="btnSubmit" value="'.$this->l('Speichern').'" class="button" /></center>
			</fieldset>
		</form>
			<p>&nbsp;</p>
		<link rel="stylesheet" type="text/css" href="'.__PS_BASE_URI__.'css/jquery.autocomplete.css" />
		<script type="text/javascript" src="'.__PS_BASE_URI__.'js/jquery/jquery.autocomplete.js"></script>
		<form action="'.$this->getConfUrl().'" method="post">
			<input type="hidden" name="post_action" value="addcatexclude" />
			<fieldset style="width: 425px; float: left"><legend><img src="../img/admin/delete.gif" alt="" title="" />'.$this->l('Kategorien ausschließen').'</legend>
				<label style="width: 60px;">'.$this->l('Hinzufügen').'</label>
				<div class="margin-form" style="padding-left: 100px;">'.$this->makeMenu('cat').' <input type="submit" name="btnSubmit" value="'.$this->l('Hinzufügen').'" class="button" />
				</div>

				'. $this->makeCatList() .'
			</fieldset>
		</form>
		<form action="'.$this->getConfUrl().'" id="product_exclude_form" method="post">
			<fieldset style="width: 425px; float: left; margin-left: 19px;"><legend><img src="../img/admin/delete.gif" alt="" title="" />'.$this->l('Produkte ausschließen').'</legend>
				<label style="width: 60px">'.$this->l('Hinzufügen').'</label>
				<div id="ajax_choose_product" style="padding:2px 6px 6px 100px; width:320px;">
					<input type="hidden" name="post_action" value="addprdexclude" />
					<input type="hidden" name="product_id" id="product_exclude_id" value="-1" />
					<input type="text" value="" id="product_autocomplete_input" />
					<input type="button" name="btnAddExclude" value="'.$this->l('Hinzufügen').'" class="button" onclick="$(this).prev().search();" />
				</div>
				<script type="text/javascript">
					function afterPrExclude(event, data, formatted){
						if(data==null) return false;
						
						$(\'#product_exclude_id\').val(data[1]);
						$(\'#product_exclude_form\').submit();
					}

					$(function() {
						$(\'#product_autocomplete_input\')
							.autocomplete(\'ajax_products_list.php\', {
								minChars: 1,
								autoFill: true,
								max:20,
								matchContains: true,
								mustMatch:true,
								scroll:false,
								cacheLength:0,
								formatItem: function(item) {
									return item[1]+\' - \'+item[0];
								}
							}).result(afterPrExclude);
						$(\'#product_autocomplete_input\').setOptions({
							extraParams: {excludeIds : ['.implode(',', $this->exclude_prd).']}
						});
					});
				</script>

				'. $this->makeProductList($this->exclude_prd) .'
			</fieldset>
		</form>

		<form action="'.$this->getConfUrl().'" method="post" id="product_include_form">
			<div class="clear">&nbsp;</div>
			<fieldset style="width: 425px;"><legend><img src="../img/admin/enabled.gif" alt="" title="" />'.$this->l('Produkte - Kostenloser Versand').'</legend>

				<label style="width: 60px">'.$this->l('Hinzufügen').'</label>
				<div id="ajax_choose_product_inc" style="padding:2px 6px 6px 100px; width:320px;">
					<input type="hidden" name="post_action" value="addprdinclude" />
					<input type="hidden" name="product_id" id="product_include_id" value="-1" />
					<input type="text" value="" id="product_autocomplete_input_inc" />
					<input type="button" name="btnAddInclude" value="'.$this->l('Hinzufügen').'" class="button" onclick="$(this).prev().search();" />
				</div>
				<script type="text/javascript">
					function afterPrInclude(event, data, formatted){
						if(data==null) return false;
						
						$(\'#product_include_id\').val(data[1]);
						$(\'#product_include_form\').submit();
					}

					$(function() {
						$(\'#product_autocomplete_input_inc\')
							.autocomplete(\'ajax_products_list.php\', {
								minChars: 1,
								autoFill: true,
								max:20,
								matchContains: true,
								mustMatch:true,
								scroll:false,
								cacheLength:0,
								formatItem: function(item) {
									return item[1]+\' - \'+item[0];
								}
							}).result(afterPrInclude);
						$(\'#product_autocomplete_input_inc\').setOptions({
							extraParams: {excludeIds : ['.implode(',', $this->include_prd).']}
						});
					});
				</script>

				'. $this->makeProductList($this->include_prd, '_incl') .'
			</fieldset>
		</form>';
	}

	private function getCatList(){
		global $cookie;

		$array = array();
		$_list = Category::getSimpleCategories(intval($cookie->id_lang));
		foreach($_list as $list) $array["$list[id_category]"] = $list['name'];

		return $array;
	}

	private function makeCatList(){
		$catlist = $this->getCatList();

		if(count($this->exclude_cat) == 0) return "";
		$list = '<ul class="cat_list">';
		foreach($this->exclude_cat as $c){
			$list .= '<li><a href="'.$this->getConfUrl().'&delete_action=1&delete_cat='.$c.'"><img src="'._PS_IMG_.'admin/disabled.gif" width="16" height="16"></a> ' . $catlist[$c] . '</li>';
		}

		$list .= '</ul>';

		return $list;
	}

	private function makeProductList($lst, $tag="_excl"){
		global $cookie;

		if(count($lst) == 0) return "";
		$list = '<ul class="product_list">';
		foreach($lst as $l){
			$p = new Product($l, false, intval($cookie->id_lang));
			$list .= '<li><a href="'.$this->getConfUrl().'&delete_action=1&delete'.$tag.'='.$l.'"><img src="'._PS_IMG_.'admin/disabled.gif" width="16" height="16"></a> ' . $p->name . '</li>';
		}
		$list .= '</ul>';

		return $list;
	}

	private function getConfUrl(){
		global $currentIndex;

		return $currentIndex.'&configure=freeshipping&token='.Tools::getValue('token');
	}

	private function _postProcess(){
		global $currentIndex;

		$this->_postErrors = array();
		if(Tools::isSubmit('post_action')){
			switch(Tools::getValue('post_action')){
				case 'save_settings':
					if (!Validate::isPrice(Tools::getValue('from-weight'))){
						$this->_postErrors[] = Tools::displayError($this->l('Ungültige Gewichtsangabe!'));
					} elseif (!Validate::isUnsignedFloat(Tools::getValue('from-money'))) {
						$this->_postErrors[] = Tools::displayError($this->l('Ungültige Betragsangabe!'));
					}

					if (!sizeof($this->_postErrors)){
						$this->free_shipping_bound = floatval(Tools::getValue('from-money'));
						$this->free_shipping_weight_bound = floatval(Tools::getValue('from-weight'));
						Configuration::updateValue('FREE_SHIPPING_BOUNDARY', $this->free_shipping_bound);
						Configuration::updateValue('FREE_SHIPPING_WEIGHT_BOUNDARY', $this->free_shipping_weight_bound);

						Tools::redirectAdmin($this->getConfUrl() . '&conf=4');
					}

					break;
				case 'addcatexclude':
					if(empty($_POST['catid']) || (int)(Tools::getValue('catid') < 1)){
						$this->_postErrors[] =  Tools::displayError($this->l('Ungültige Kategorie ID'));
						return;
					}
					
					$cat = (int)(Tools::getValue('catid'));
					if(!in_array($cat, $this->exclude_cat)){
						$this->exclude_cat[] = $cat;
						Configuration::updateValue('FREE_SHIPPING_EXCLUDE_CAT', serialize($this->exclude_cat));
					}

					Tools::redirectAdmin($this->getConfUrl() . '&conf=3');

					break;
				case 'addprdinclude':
					if(empty($_POST['product_id']) || (int)(Tools::getValue('product_id') < 1)){
						$this->_postErrors[] =  Tools::displayError($this->l('Ungültige Produkt ID'));
						return;
					}

					$pid = (int)(Tools::getValue('product_id'));
					if(!in_array($pid, $this->include_prd)){
						$this->include_prd[] = $pid;
						Configuration::updateValue('FREE_SHIPPING_INCLUDE_PRD', serialize($this->include_prd));
					}

					Tools::redirectAdmin($this->getConfUrl() . '&conf=3');

					break;
				case 'addprdexclude':
					if(empty($_POST['product_id']) || (int)(Tools::getValue('product_id') < 1)){
						$this->_postErrors[] =  Tools::displayError($this->l('Ungültige Produkt ID'));
						return;
					}

					$pid = (int)(Tools::getValue('product_id'));
					if(!in_array($pid, $this->exclude_prd)){
						$this->exclude_prd[] = $pid;
						Configuration::updateValue('FREE_SHIPPING_EXCLUDE_PRD', serialize($this->exclude_prd));
					}

					Tools::redirectAdmin($this->getConfUrl() . '&conf=3');

					break;
			}
		}
	}

	private function _getProcess(){
		global $currentIndex;
		if(isset($_GET['delete_cat'])){
			$cat = (int)(Tools::getValue('delete_cat'));
			$this->removeFromList(&$this->exclude_cat, $cat);
			Configuration::updateValue('FREE_SHIPPING_EXCLUDE_CAT', serialize($this->exclude_cat));
		} elseif(isset($_GET['delete_incl'])){
			$pid = (int)(Tools::getValue('delete_incl'));
			$this->removeFromList(&$this->include_prd, $pid);
			Configuration::updateValue('FREE_SHIPPING_INCLUDE_PRD', serialize($this->include_prd));
		} elseif(isset($_GET['delete_excl'])){
			$pid = (int)(Tools::getValue('delete_excl'));
			$this->removeFromList(&$this->exclude_prd, $pid);
			Configuration::updateValue('FREE_SHIPPING_EXCLUDE_PRD', serialize($this->exclude_prd));

		}

		Tools::redirectAdmin($this->getConfUrl() . '&conf=1');
	}

	private function removeFromList($list, $id){
		foreach($list as $key => $value){
			if($value == $id || $value < 1) unset($list[$key]);
		}
	}

	public function getContent() {
		$this->_html = '<h2>'.$this->displayName.'</h2>';

		if(isset($_GET['delete_action'])){
			$this->_getProcess();
		}

		if (!empty($_POST))
		{
			$this->_postProcess();

			if (sizeof($this->_postErrors))
				foreach ($this->_postErrors AS $err)
					$this->_html .= '<div class="alert error">'. $err .'</div>';
		}

		$this->_displayForm();
		return $this->_html;
	}
	
}